// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package upb_test;

import "upb/test/test.proto";
import "upb/test/test_cpp.proto";

message ContainsImported {
  optional MessageName message_name = 1;
  optional upb.test.TestMessage test_message = 2;
}
