// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto3";

package pkg;

message RegularImportMessage {}

enum Proto3Enum {
  PROTO3_ENUM_ZERO = 0;
}
